unit ULogic;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, math, Graphics;

const
  Ueben = 'Naja, da muss wohl noch ein wenig geübt werden ;)';
  Super = 'Gar nicht schlecht, eine wirklich gute Leistung!';
  Spitze = 'Unglaublich, dass Du das geschafft hast! TipTop!';
  Langweilig = 'Ist Dir irgendwie langweilig?';

type
  TRichtung = (Rechts, Links, Hoch, Runter);

  THaeppchen = record
    RestTime: Integer;
    x: Integer;
    y: Integer;
  end;

  { TFutter }

  TFutter = class(TObject)
  private
    FHaeppchen: array of THaeppchen;
    procedure Delete(Index: Integer);
  public
    procedure Add(P: TPoint);
    procedure Welken;
    procedure Clear;
    function AufPunkt(P: TPoint; out Index: Integer): Boolean;
    function Fressen(P: TPoint): Boolean;
    function GetColor(Index: Integer): TColor;
  end;

  { TSchlange }

  TSchlange = class(TObject)
  private
    FFeldMaxX: Integer;
    FFeldMaxY: Integer;
    FRichtung: TList;
    FKoordinaten: array of TPoint;
    FScore: Integer;
    function GetKopf: TPoint;
    function GetRichtung: TRichtung;
    procedure SetRichtung(ARichtung: TRichtung);
  public
    constructor Create;
    destructor Destroy; override;
    function IndexOf(P: TPoint): Integer;
    function GetColor(Index: Integer): TColor;
    procedure Init;
    procedure Bewegung;
    procedure ErhoeheScore(Punkte: Integer);
    procedure Wachsen;
    function Kollision: Boolean;
    function AufPunkt(P: TPoint; MitKopf: Boolean = True): Boolean;
    property Kopf: TPoint read GetKopf;
    property Richtung: TRichtung read GetRichtung write SetRichtung;
    property FeldMaxX: Integer read FFeldMaxX write FFeldMaxX;
    property FeldMaxY: Integer read FFeldMaxY write FFeldMaxY;
    property Score: Integer read FScore;
  end;

implementation

{ TFutter }

procedure TFutter.Delete(Index: Integer);
var
  i: Integer;
begin
  for i := Index to High(FHaeppchen) do
    FHaeppchen[i] := FHaeppchen[i + 1];
  SetLength(FHaeppchen, Length(FHaeppchen) - 1);
end;

procedure TFutter.Add(P: TPoint);
begin
  SetLength(FHaeppchen, Length(FHaeppchen) + 1);
  FHaeppchen[High(FHaeppchen)].RestTime := 100;
  FHaeppchen[High(FHaeppchen)].x := P.x;
  FHaeppchen[High(FHaeppchen)].y := P.y;
end;

procedure TFutter.Welken;
var
  i: Integer;
begin
  for i := High(FHaeppchen) downto 0 do begin
    Dec(FHaeppchen[i].RestTime);
    if FHaeppchen[i].RestTime = 0 then
      Delete(i);
  end;
end;

procedure TFutter.Clear;
var
  i: Integer;
begin
  for i := High(FHaeppchen) downto 0 do Delete(i);
end;

function TFutter.AufPunkt(P: TPoint; out Index: Integer
  ): Boolean;
begin
  Result := False;
  Index := 0;
  while Index < Length(FHaeppchen) do
    if (FHaeppchen[Index].x = P.x) and (FHaeppchen[Index].y = P.y) then begin
      Result := True;
      Exit;
    end else
      Inc(Index);
  Result := False;
  Index  := -1;
end;

function TFutter.Fressen(P: TPoint): Boolean;
var
  i: Integer;
begin
  if not AufPunkt(P, i) then Exit(False);
  Delete(i);
  Result := True;
end;

function TFutter.GetColor(Index: Integer): TColor;
var
  Part: Byte;
begin
  Part := 255 - Min(FHaeppchen[Index].RestTime * 10, 255);
  Result := RGBToColor(Part, 128 + Part div 2, Part);
end;

{ TSchlange }

function TSchlange.GetKopf: TPoint;
begin
  Result := FKoordinaten[0];
end;

function TSchlange.GetRichtung: TRichtung;
begin
  if FRichtung.Count = 0 then Exit(Rechts);
  Result := TRichtung({%H-}Integer(FRichtung.Items[0]));
  if FRichtung.Count = 1 then Exit;
  Result := TRichtung({%H-}Integer(FRichtung.Items[1]));
  FRichtung.Delete(0);
end;

procedure TSchlange.SetRichtung(ARichtung: TRichtung);
begin
  FRichtung.Add({%H-}Pointer(Integer(ARichtung)));
end;

constructor TSchlange.Create;
begin
  inherited Create;
  FRichtung := TList.Create;
  Init;
end;

destructor TSchlange.Destroy;
begin
  FreeAndNil(FRichtung);
  inherited Destroy;
end;

function TSchlange.IndexOf(P: TPoint): Integer;
var
  i: Integer;
begin
  for i := 0 to High(FKoordinaten) do
    if (FKoordinaten[i].x = P.x) and (FKoordinaten[i].y = P.y) then
      Exit(i);
  Result := -1;
end;

function TSchlange.GetColor(Index: Integer): TColor;
begin
  if Index = 0 then Exit(clRed);
  Result := RGBToColor(240, 120 ,50 + Trunc(Sin(Index) * 50));
end;

procedure TSchlange.Init;
var
  i: Integer;
begin
  SetLength(FKoordinaten, 3);
  for i := 0 to High(FKoordinaten) do begin
    FKoordinaten[i].x := - i;
    FKoordinaten[i].y := 0;
  end;
  FRichtung.Clear;
  Richtung := Rechts;
  FScore   := 0;
end;

procedure TSchlange.Bewegung;
var
  i: Integer;
begin
  for i := High(FKoordinaten) downto 1 do
    FKoordinaten[i] := FKoordinaten[i - 1];
  case Richtung of
    Links:  FKoordinaten[0].x := FKoordinaten[0].x - 1;
    Rechts: FKoordinaten[0].x := FKoordinaten[0].x + 1;
    Hoch:   FKoordinaten[0].y := FKoordinaten[0].y - 1;
    Runter: FKoordinaten[0].y := FKoordinaten[0].y + 1;
  end;
end;

procedure TSchlange.ErhoeheScore(Punkte: Integer);
begin
  FScore := FScore + Punkte + Length(FKoordinaten) div 10;
end;

procedure TSchlange.Wachsen;
begin
  SetLength(FKoordinaten, Length(FKoordinaten) + 1);
  FKoordinaten[High(FKoordinaten)].x := -1;
  FKoordinaten[High(FKoordinaten)].y := -1;
end;

function TSchlange.Kollision: Boolean;
begin
  if AufPunkt(Kopf, False) then Exit(True);
  if FKoordinaten[0].x < 0 then Exit(True);
  if FKoordinaten[0].y < 0 then Exit(True);
  if FKoordinaten[0].x > FeldMaxX then Exit(True);
  if FKoordinaten[0].y > FeldMaxY then Exit(True);
  Result := False;
end;

function TSchlange.AufPunkt(P: TPoint; MitKopf: Boolean
  ): Boolean;
var
  i: Integer;
begin
  Result := false;
  if MitKopf then
    i := 0
  else
    i := 1;
  while i < Length(FKoordinaten) do
    if (FKoordinaten[i].x = P.x) and (FKoordinaten[i].y = P.y) then
      Exit(true)
    else
      Inc(i);
end;

end.



