unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls,
  Graphics, Dialogs, Grids, ExtCtrls, StdCtrls, LCLType,
  ULogic;

type

  { TfrmMain }

  TfrmMain = class(TForm)
    btnNeuesSpiel: TButton;
    btnPause: TButton;
    Spielfeld: TDrawGrid;
    timGeschwindigkeit: TTimer;
    timSpielSchritt: TTimer;
    timFutter: TTimer;
    procedure btnNeuesSpielClick({%H-}Sender: TObject);
    procedure btnPauseClick({%H-}Sender: TObject);
    procedure SpielfeldDrawCell({%H-}Sender: TObject; aCol,
      aRow: Integer; aRect: TRect; {%H-}aState: TGridDrawState);
    procedure SpielfeldEnter({%H-}Sender: TObject);
    procedure FormChangeBounds({%H-}Sender: TObject);
    procedure FormCreate({%H-}Sender: TObject);
    procedure FormDestroy({%H-}Sender: TObject);
    procedure FormKeyDown({%H-}Sender: TObject; var Key: Word;
      {%H-}Shift: TShiftState);
    procedure timGeschwindigkeitTimer({%H-}Sender: TObject);
    procedure timSpielSchrittTimer({%H-}Sender: TObject);
    procedure timFutterTimer({%H-}Sender: TObject);
  private
    Schlange: TSchlange;
    Futter: TFutter;
    procedure InitSpiel({%H-}Data: PtrInt);
    procedure SpielEnde;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.lfm}

{ TfrmMain }

procedure TfrmMain.FormChangeBounds(Sender: TObject);
begin
  Application.QueueAsyncCall(@InitSpiel, 0);
end;

procedure TfrmMain.SpielfeldDrawCell(Sender: TObject; aCol,
  aRow: Integer; aRect: TRect; aState: TGridDrawState);
var
  aIndex: Integer;
begin
  if Schlange.AufPunkt(Point(aCol, aRow)) then begin
    Spielfeld.Canvas.Brush.Color := Schlange.GetColor(Schlange.IndexOf(Point(aCol, aRow)));
    Spielfeld.Canvas.FillRect(aRect);
    Exit;
  end;
  if Futter.AufPunkt(Point(aCol, aRow), aIndex) then begin
    Spielfeld.Canvas.Brush.Color := Futter.GetColor(aIndex);
    Spielfeld.Canvas.FillRect(aRect);
  end;
end;

procedure TfrmMain.SpielfeldEnter(Sender: TObject);
begin
  btnPause.SetFocus;
end;

procedure TfrmMain.btnNeuesSpielClick(Sender: TObject);
begin
  InitSpiel(0);
  Repaint;
end;

procedure TfrmMain.btnPauseClick(Sender: TObject);
begin
  timSpielSchritt.Enabled    := not timSpielSchritt.Enabled;
  timFutter.Enabled          := timSpielSchritt.Enabled;
  timGeschwindigkeit.Enabled := timSpielSchritt.Enabled;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Randomize;
  Schlange := TSchlange.Create;
  Futter   := TFutter.Create;
  Left   := Screen.Width div 4;
  Top    := Screen.Height div 4;
  Width  := Screen.Width div 2;
  Height := Screen.Height div 2;
  Constraints.MinHeight := Screen.Height div 4;
  Constraints.MinWidth  := Constraints.MinHeight;
  Spielfeld.DefaultColWidth  := Screen.Width div 80;
  Spielfeld.DefaultRowHeight := Spielfeld.DefaultColWidth;
  Application.QueueAsyncCall(@InitSpiel, 0);
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  Futter.Free;
  Schlange.Free;
end;

procedure TfrmMain.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_LEFT,  VK_A: Schlange.Richtung := Links;
    VK_RIGHT, VK_D: Schlange.Richtung := Rechts;
    VK_UP,    VK_W: Schlange.Richtung := Hoch;
    VK_DOWN,  VK_S: Schlange.Richtung := Runter;
    VK_SPACE      : btnPauseClick(nil);
    else Exit;
  end;
  Key := 0;
end;

procedure TfrmMain.timGeschwindigkeitTimer(Sender: TObject);
begin
  timSpielSchritt.Interval := timSpielSchritt.Interval - 1;
end;

procedure TfrmMain.timSpielSchrittTimer(Sender: TObject);
begin
  Schlange.Bewegung;
  if Schlange.Kollision then
    Spielende;
  if Futter.Fressen(Schlange.Kopf) then begin
    Schlange.ErhoeheScore(10);
    Schlange.Wachsen;
  end;
  Schlange.ErhoeheScore(1);
  Repaint;
  Caption := 'Gib der Schlange Futter - Punkte: ' + IntToStr(Schlange.Score);
end;

procedure TfrmMain.timFutterTimer(Sender: TObject);
var
  aPoint: TPoint;
begin
  Futter.Welken;
  if Random(10) = 0 then begin
    repeat
      aPoint.x := Random(Spielfeld.ColCount);
      aPoint.y := Random(Spielfeld.RowCount);
    until not Schlange.AufPunkt(aPoint);
    Futter.Add(aPoint);
  end;
end;

procedure TfrmMain.InitSpiel(Data: PtrInt);
begin
  Spielfeld.ColCount := Spielfeld.ClientWidth div Spielfeld.DefaultColWidth;
  Schlange.FeldMaxX  := Spielfeld.ColCount - 1;
  Spielfeld.RowCount := Spielfeld.ClientHeight div Spielfeld.DefaultRowHeight;
  Schlange.FeldMaxY  := Spielfeld.RowCount - 1;
  btnPause.Width := ClientWidth div 2 - 15;
  Schlange.Init;
  Futter.Clear;
  timSpielSchritt.Enabled    := False;
  timFutter.Enabled          := False;
  timGeschwindigkeit.Enabled := False;
  timSpielSchritt.Interval   := 250;
  Caption := 'Gib der Schlange Futter - Spielstart mit [Space]';
end;

procedure TfrmMain.SpielEnde;
var
  Inhalt: String;
begin
  timSpielSchritt.Enabled := False;
  if Schlange.Score < 1000 then Inhalt := Ueben
  else if Schlange.Score < 5000 then Inhalt := Super
  else if Schlange.Score < 20000 then Inhalt := Spitze
  else Inhalt := Langweilig;
  case QuestionDLG(IntToStr(Schlange.Score) + ' Punkte',
      Inhalt + LineEnding + LineEnding + 'Willst Du ein neues Spiel beginnen?',
      mtConfirmation,
      [mrYes, 'Ja', mrNo, 'Nein'], 0) of
    mrYes: InitSpiel(0);
    mrNo:  Close;
  end;
end;

end.

